package at.ac.uibk.gitsearch.service;

import java.io.IOException;
import java.io.InputStream;

import org.codeability.sharing.plugins.api.SharingPluginConfig;
import org.codeability.sharing.plugins.api.SharingPluginConfig.Action;
import org.junit.Assert;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.util.FileCopyUtils;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.yaml.YAMLFactory;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.service.PluginManagementService.PluginConfigWrapper;
import at.ac.uibk.gitsearch.service.PluginManagementService.PluginConfigWrapper.ActionWrapper;
import at.ac.uibk.gitsearch.service.dto.SearchResultDTO;
import at.ac.uibk.gitsearch.service.dto.UserProvidedMetadataDTO;

@SpringBootTest(classes = GitsearchApp.class)
public class PluginManagerActionIT {

	private final Logger log = LoggerFactory.getLogger(PluginManagerActionIT.class);
	private PluginConfigWrapper config;

	@BeforeEach
	public void initConfig() {
		config = new PluginConfigWrapper(
				new SharingPluginConfig("TestPlugin", new Action[] { new Action("someId", "someImportURL",
						"someActionName", "metadata.type.externalName=='programming exercise'") }),
				"http://localhost:8080/xxx");
	}

	@Test
	public void testExpressionFiltersOnMetaData() throws JsonParseException, JsonMappingException, IOException {

		ActionWrapper actionWrapper = config.getActions().get("someId");

		Assert.assertTrue("metaData1 should be applicable",
				actionWrapper.isApplicable(readTestResults("metaData1.yaml")));
		Assert.assertFalse("metaData2 should not be applicable",
				actionWrapper.isApplicable(readTestResults("metaData2.yaml")));

	}

	/**
	 * temporary static test data.
	 *
	 * @param infoString just a string to add to testdata description.
	 * @return
	 * @throws IOException
	 * @throws JsonMappingException
	 * @throws JsonParseException
	 */
	private SearchResultDTO readTestResults(String testFile)
			throws JsonParseException, JsonMappingException, IOException {
		ObjectMapper mapper = new ObjectMapper(new YAMLFactory());
		mapper.findAndRegisterModules();

		log.debug("reading test data from {} ", testFile);

		final String resourceName = "./testData/" + testFile;
		final InputStream resourceStream = this.getClass().getResourceAsStream(resourceName);
		Assert.assertNotNull("cannot find resource at " + resourceName, resourceStream);

		byte[] fileContent = FileCopyUtils.copyToByteArray(resourceStream);
		UserProvidedMetadataDTO metaData = mapper.readValue(fileContent, UserProvidedMetadataDTO.class);
		SearchResultDTO result = new SearchResultDTO();
		result.setMetadata(metaData);
		return result;

	}

}
